###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__author__ = '$Author: morten $'
__version__ = '$Revision: 1.3 $'


import nntplib
import gateway
import rfc822_message_parse
import cStringIO
import string
import thread

class nntp_gateway(gateway.gateway):
    
    def __init__(self,server,port=None,timeout=2000):
        gateway.gateway.__init__(self,server)

        self.groups = {}
        self.subscribed_groups = {}

        thread.start_new_thread(self.update_groups,('000000','000000'))
        

    def update_groups(self,date,time):

        if self._update_groups_running:

            return

        self._update_groups_running = time.time()

        connection = self.build_connection()

        new_groups = connection.newgroups(date,time)[-1]

        for group_entry in new_groups:

            group = string.split(group_entry)[0]
            
            group_path = string.split(group,'.')

            ref = self.groups

            for element in group_path:

                if not ref.has_key(element):

                    ref[element] = {}

                ref = ref[element]

        self._update_groups_running = 0


    def get_group(self,group):
        """Gets a reference to the given group."""

        ref = self.groups

        for path in string.split(group,'.'):

            try:

                print path
                ref = ref[path]

            except KeyError:

                raise ValueError,'Unknown Newsgroup'

        return ref


    def verify_group(self,group):
        """Verifies newsgroups."""

        try:

            self.get_group(group)
            return 1
            
        except ValueError:

            return 0


    def subscribe_group(self,group):
        """Subscribes to given group."""

        self.verify_group(group)

        self.subscribed_groups[group] = {'timestamp':int(time.time())}

        thread.start_new_thread(self.retrieve_group,(group,))


    def retrieve_group(self,group,since=86400*31):
        """Retrieves all messages submitted since parameter since."""

        connection = self.build_connection()

        messages = []

        if not self.verify_group(group):

            return

        group_info = connection.group(group)

        first,last = map(int,group_info[2:-1])

        for i in range(first,last):

            messages.append(connection.article(str(i)))

        return messages


    def build_connection(self):
        """Returns a connection to the newsserver."""

        if self.port:

            connection = nntplib.NNTP(self.server,self.port)

        else:

            connection = nntplib.NNTP(self.server,self.port)

        return connection


    def post(self,message):
        """Deconstructs an rfc822_message object and posts it."""

        connection = self.build_connection()

        connection.post(cStringIO.StringIO(message.raw_message()))


if __name__ == '__main__':
    nntp = nntp_gateway('10.230.8.2')
