###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

__author__ = '$Author: morten $'
__version__ = '$Revision: 1.7 $'


import string
import mimetools
import multifile
import rfc822_message_base
import rfc822_attachment_base
import mime_message
import util


def rfc822_message_parse(msg):
	"""Completely deconstructs an RFC822 type message and returns
	an rfc822_message object.

	msg has to be a file-like object."""

	message = mimetools.Message(msg)

	headers = {}
	msgfrom = message.getaddr('from')
	to = message.getaddrlist('to')
	cc = message.getaddrlist('cc')
	bcc = message.getaddrlist('bcc')
	reply_to = message.getaddrlist('reply-to')
	message_id = ''
	date = None
	newsgroups = []
	parts = []
	body = ''
	attachments = []

	boundary = message.getparam('boundary') or ''

	for key in message.keys():

		headers[string.lower(key)] = message[key]

	if headers.has_key('newsgroups'):

		newsgroups = headers['newsgroups']
		newsgroups = map(string.strip,string.split(newsgroups,','))

	if headers.has_key('date'):

		date = headers['date']

	if headers.has_key('message-id'):

		message_id = headers['message-id']

	for i in headers.keys():

		if i in ('from','to','reply-to','cc','bcc','newsgroups', 'date',
				 'content-type','message-id'):

			del headers[i]


	# We're preparing the message for retrieving the body.

	msg.seek(message.startofbody)

	# If we've got a multipart message, there must also be a boundary.

	if boundary:

		multi_message = multifile.MultiFile(msg)
		multi_message.push(boundary)

		while multi_message.next():

			parsed_attachment = mime_message.mime_part_parse(multi_message)

			parts.append(parsed_attachment)

		multi_message.pop()

		# If the first part is a text-based
		# message, we'll assume it's the body of
		# the message.

		if parts[0].media_type == 'text':

			body = parts[0].data
			parts = parts[1:]

		# In any case, the rest of the parts are attachments.

		map(attachments.append,parts)

	else:

		_body = mime_message.mime_part(message.fp.readlines())

		body = _body.data

	mymsg = rfc822_message_base.rfc822_message_base(sender = msgfrom, to = to,
													cc = cc, bcc = bcc,
													reply_to = reply_to,
													date = date,
													newsgroups = newsgroups,
													message_id = message_id,
													boundary = boundary,
													headers = headers,
													body = body,
													attachments = parts)

	#mymsg._raw_message_debug = _raw_message_debug

	return mymsg
